@@ -1,5 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-"""Supernova remnant (SNR) source models"""
+"""Supernova remnant (SNR) source models."""
 from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 from astropy.units import Quantity
@@ -7,10 +7,7 @@ import astropy.constants as const
 from astropy.utils import lazyproperty
 from ...extern.validator import validate_physical_type
 
-__all__ = [
-    'SNR',
-    'SNRTrueloveMcKee',
-]
+__all__ = ["SNR", "SNRTrueloveMcKee"]
 
 
 class SNR(object):
@@ -34,10 +31,17 @@ class SNR(object):
         Post-shock temperature where gamma-ray emission stops.
     """
 
-    def __init__(self, e_sn=Quantity(1e51, 'erg'), theta=Quantity(0.1),
-                 n_ISM=Quantity(1, 'cm-3'), m_ejecta=const.M_sun,
-                 t_stop=Quantity(1e6, 'K'), age=None, morphology='Shell2D',
-                 spectral_index=2.1):
+    def __init__(
+        self,
+        e_sn=Quantity(1e51, "erg"),
+        theta=Quantity(0.1),
+        n_ISM=Quantity(1, "cm-3"),
+        m_ejecta=const.M_sun,
+        t_stop=Quantity(1e6, "K"),
+        age=None,
+        morphology="Shell2D",
+        spectral_index=2.1,
+    ):
         self.e_sn = e_sn
         self.theta = theta
         self.rho_ISM = n_ISM * const.m_p
@@ -47,7 +51,7 @@ class SNR(object):
         self.morphology = morphology
         self.spectral_index = spectral_index
         if age is not None:
-            validate_physical_type('age', age, 'time')
+            validate_physical_type("age", age, "time")
             self.age = age
 
     def radius(self, t=None):
@@ -76,15 +80,17 @@ class SNR(object):
 
         """
         if t is not None:
-            validate_physical_type('t', t, 'time')
-        elif hasattr(self, 'age'):
+            validate_physical_type("t", t, "time")
+        elif hasattr(self, "age"):
             t = self.age
         else:
-            raise ValueError('Need time variable or age attribute.')
-        r = np.where(t > self.sedov_taylor_begin,
-                     self._radius_sedov_taylor(t).to('cm').value,
-                     self._radius_free_expansion(t).to('cm').value)
-        return Quantity(r, 'cm')
+            raise ValueError("Need time variable or age attribute.")
+        r = np.where(
+            t > self.sedov_taylor_begin,
+            self._radius_sedov_taylor(t).to("cm").value,
+            self._radius_free_expansion(t).to("cm").value,
+        )
+        return Quantity(r, "cm")
 
     def _radius_free_expansion(self, t):
         """Shock radius at age t during free expansion phase.
@@ -96,9 +102,9 @@ class SNR(object):
 
         """
         # proportional constant for the free expansion phase
-        term_1 = (self.e_sn / Quantity(1e51, 'erg')) ** (1. / 2)
+        term_1 = (self.e_sn / Quantity(1e51, "erg")) ** (1. / 2)
         term_2 = (self.m_ejecta / const.M_sun) ** (-1. / 2)
-        return Quantity(0.01, 'pc/yr') * term_1 * term_2 * t
+        return Quantity(0.01, "pc/yr") * term_1 * term_2 * t
 
     def _radius_sedov_taylor(self, t):
         """Shock radius  at age t  during Sedov Taylor phase.
@@ -123,7 +129,7 @@ class SNR(object):
         """
         return self.radius(t) * (1 - fraction)
 
-    def luminosity_tev(self, t=None, energy_min=Quantity(1, 'TeV')):
+    def luminosity_tev(self, t=None, energy_min=Quantity(1, "TeV")):
         """Gamma-ray luminosity above ``energy_min`` at age ``t``.
 
         The luminosity is assumed constant in a given age interval and zero
@@ -151,21 +157,23 @@ class SNR(object):
 
         """
         if t is not None:
-            validate_physical_type('t', t, 'time')
-        elif hasattr(self, 'age'):
+            validate_physical_type("t", t, "time")
+        elif hasattr(self, "age"):
             t = self.age
         else:
-            raise ValueError('Need time variable or age attribute.')
+            raise ValueError("Need time variable or age attribute.")
 
         # Flux in 1 k distance according to Drury formula 9
-        term_0 = energy_min / Quantity(1, 'TeV')
-        term_1 = self.e_sn / Quantity(1e51, 'erg')
-        term_2 = self.rho_ISM / (Quantity(1, 'cm-3') * const.m_p)
+        term_0 = energy_min / Quantity(1, "TeV")
+        term_1 = self.e_sn / Quantity(1e51, "erg")
+        term_2 = self.rho_ISM / (Quantity(1, "cm-3") * const.m_p)
         L = self.theta * term_0 ** (1 - self.spectral_index) * term_1 * term_2
 
         # Corresponding luminosity
-        L = np.select([t <= self.sedov_taylor_begin, t <= self.sedov_taylor_end], [0, L])
-        return Quantity(1.0768E34, 's-1') * L
+        L = np.select(
+            [t <= self.sedov_taylor_begin, t <= self.sedov_taylor_end], [0, L]
+        )
+        return Quantity(1.0768E34, "s-1") * L
 
     @lazyproperty
     def sedov_taylor_begin(self):
@@ -185,10 +193,10 @@ class SNR(object):
             \\left(\\frac{\\rho_{ISM}}{10^{-24}g/cm^3}\\right)^{-1/3}
 
         """
-        term1 = (self.e_sn / Quantity(1e51, 'erg')) ** (-1. / 2)
+        term1 = (self.e_sn / Quantity(1e51, "erg")) ** (-1. / 2)
         term2 = (self.m_ejecta / const.M_sun) ** (5. / 6)
-        term3 = (self.rho_ISM / (Quantity(1, 'cm-3') * const.m_p)) ** (-1. / 3)
-        return Quantity(200, 'yr') * term1 * term2 * term3
+        term3 = (self.rho_ISM / (Quantity(1, "cm-3") * const.m_p)) ** (-1. / 3)
+        return Quantity(200, "yr") * term1 * term2 * term3
 
     @lazyproperty
     def sedov_taylor_end(self):
@@ -209,7 +217,7 @@ class SNR(object):
         """
         term1 = 3 * const.m_p.cgs / (100 * const.k_B.cgs * self.t_stop)
         term2 = (self.e_sn / self.rho_ISM) ** (2. / 5)
-        return ((term1 * term2) ** (5. / 6)).to('yr')
+        return ((term1 * term2) ** (5. / 6)).to("yr")
 
 
 class SNRTrueloveMcKee(SNR):
@@ -223,7 +231,11 @@ class SNRTrueloveMcKee(SNR):
 
         # Characteristic dimensions
         self.r_c = self.m_ejecta ** (1. / 3) * self.rho_ISM ** (-1. / 3)
-        self.t_c = self.e_sn ** (-1. / 2) * self.m_ejecta ** (5. / 6) * self.rho_ISM ** (-1. / 3)
+        self.t_c = (
+            self.e_sn ** (-1. / 2)
+            * self.m_ejecta ** (5. / 6)
+            * self.rho_ISM ** (-1. / 3)
+        )
 
     def radius(self, t=None):
         """Outer shell radius at age t.
@@ -257,19 +269,19 @@ class SNRTrueloveMcKee(SNR):
 
         """
         if t is not None:
-            validate_physical_type('t', t, 'time')
-        elif hasattr(self, 'age'):
+            validate_physical_type("t", t, "time")
+        elif hasattr(self, "age"):
             t = self.age
         else:
-            raise ValueError('Need time variable or age attribute.')
+            raise ValueError("Need time variable or age attribute.")
 
         # Evaluate `_radius_sedov_taylor` on `t > self.sedov_taylor_begin`
         # only to avoid a warning
         r = np.empty(t.shape, dtype=np.float64)
         mask = t > self.sedov_taylor_begin
-        r[mask] = self._radius_sedov_taylor(t[mask]).to('cm').value
-        r[~mask] = self._radius_free_expansion(t[~mask]).to('cm').value
-        return Quantity(r, 'cm')
+        r[mask] = self._radius_sedov_taylor(t[mask]).to("cm").value
+        r[~mask] = self._radius_free_expansion(t[~mask]).to("cm").value
+        return Quantity(r, "cm")
 
     def _radius_free_expansion(self, t):
         """Shock radius at age t during free expansion phase.
@@ -327,20 +339,18 @@ class SNRTrueloveMcKee(SNR):
             \\ln \\left(\\frac{t}{t_{core}}\\right)\\right]\\frac{R_{ch}}{t_{ch}}t
         """
         if t is not None:
-            validate_physical_type('t', t, 'time')
-        elif hasattr(self, 'age'):
+            validate_physical_type("t", t, "time")
+        elif hasattr(self, "age"):
             t = self.age
         else:
-            raise ValueError('Need time variable or age attribute.')
+            raise ValueError("Need time variable or age attribute.")
 
         # Time when reverse shock reaches the "core"
         t_core = 0.25 * self.t_c
 
         term1 = (t - t_core) / (self.t_c)
-        term2 = (1.49 - 0.16 * term1 - 0.46 * np.log(t / t_core))
+        term2 = 1.49 - 0.16 * term1 - 0.46 * np.log(t / t_core)
         R_1 = self._radius_free_expansion(t) / 1.19
         R_RS = term2 * (self.r_c / self.t_c) * t
-        r = np.where(t < t_core,
-                     R_1.to('cm').value,
-                     R_RS.to('cm').value)
-        return Quantity(r, 'cm')
+        r = np.where(t < t_core, R_1.to("cm").value, R_RS.to("cm").value)
+        return Quantity(r, "cm")
