@@ -8,38 +8,47 @@ from astropy import units as u
 from astropy.io.registry import IORegistryError
 from ..utils.scripts import make_path
 from ..utils.table import table_standardise_units_copy, table_from_row_data
+from ..utils.fitting import Fit
+from ..stats.fit_statistics import chi2
 from .models import PowerLaw
 from .powerlaw import power_law_integral_flux
+from . import SpectrumObservationList, SpectrumObservation
 
 __all__ = [
-    'FluxPoints',
+    "FluxPoints",
     # 'FluxPointProfiles',
-    'FluxPointEstimator',
-    'FluxPointFitter',
+    "FluxPointEstimator",
+    "FluxPointFit",
 ]
 
 log = logging.getLogger(__name__)
 
-REQUIRED_COLUMNS = OrderedDict([
-    ('dnde', ['e_ref', 'dnde']),
-    ('e2dnde', ['e_ref', 'e2dnde']),
-    ('flux', ['e_min', 'e_max', 'flux']),
-    ('eflux', ['e_min', 'e_max', 'eflux']),
-])
+REQUIRED_COLUMNS = OrderedDict(
+    [
+        ("dnde", ["e_ref", "dnde"]),
+        ("e2dnde", ["e_ref", "e2dnde"]),
+        ("flux", ["e_min", "e_max", "flux"]),
+        ("eflux", ["e_min", "e_max", "eflux"]),
+    ]
+)
 
-OPTIONAL_COLUMNS = OrderedDict([
-    ('dnde', ['dnde_err', 'dnde_errp', 'dnde_errn', 'dnde_ul', 'is_ul']),
-    ('e2dnde', ['e2dnde_err', 'e2dnde_errp', 'e2dnde_errn', 'e2dnde_ul', 'is_ul']),
-    ('flux', ['flux_err', 'flux_errp', 'flux_errn', 'flux_ul', 'is_ul']),
-    ('eflux', ['eflux_err', 'eflux_errp', 'eflux_errn', 'eflux_ul', 'is_ul']),
-])
+OPTIONAL_COLUMNS = OrderedDict(
+    [
+        ("dnde", ["dnde_err", "dnde_errp", "dnde_errn", "dnde_ul", "is_ul"]),
+        ("e2dnde", ["e2dnde_err", "e2dnde_errp", "e2dnde_errn", "e2dnde_ul", "is_ul"]),
+        ("flux", ["flux_err", "flux_errp", "flux_errn", "flux_ul", "is_ul"]),
+        ("eflux", ["eflux_err", "eflux_errp", "eflux_errn", "eflux_ul", "is_ul"]),
+    ]
+)
 
-DEFAULT_UNIT = OrderedDict([
-    ('dnde', u.Unit('cm-2 s-1 TeV-1')),
-    ('e2dnde', u.Unit('erg cm-2 s-1')),
-    ('flux', u.Unit('cm-2 s-1')),
-    ('eflux', u.Unit('erg cm-2 s-1')),
-])
+DEFAULT_UNIT = OrderedDict(
+    [
+        ("dnde", u.Unit("cm-2 s-1 TeV-1")),
+        ("e2dnde", u.Unit("erg cm-2 s-1")),
+        ("flux", u.Unit("cm-2 s-1")),
+        ("eflux", u.Unit("erg cm-2 s-1")),
+    ]
+)
 
 
 class FluxPoints(object):
@@ -151,12 +160,12 @@ class FluxPoints(object):
         try:
             table = Table.read(str(filename), **kwargs)
         except IORegistryError:
-            kwargs.setdefault('format', 'ascii.ecsv')
+            kwargs.setdefault("format", "ascii.ecsv")
             table = Table.read(str(filename), **kwargs)
 
-        if 'SED_TYPE' not in table.meta.keys():
+        if "SED_TYPE" not in table.meta.keys():
             sed_type = cls._guess_sed_type(table)
-            table.meta['SED_TYPE'] = sed_type
+            table.meta["SED_TYPE"] = sed_type
 
         return cls(table=table)
 
@@ -174,7 +183,7 @@ class FluxPoints(object):
         try:
             self.table.write(str(filename), **kwargs)
         except IORegistryError:
-            kwargs.setdefault('format', 'ascii.ecsv')
+            kwargs.setdefault("format", "ascii.ecsv")
             self.table.write(str(filename), **kwargs)
 
     @classmethod
@@ -206,7 +215,7 @@ class FluxPoints(object):
             tables.append(table[reference.colnames])
 
         table_stacked = vstack(tables)
-        table_stacked.meta['SED_TYPE'] = reference.meta['SED_TYPE']
+        table_stacked.meta["SED_TYPE"] = reference.meta["SED_TYPE"]
 
         return cls(table_stacked)
 
@@ -229,10 +238,10 @@ class FluxPoints(object):
         >>> print(flux_points.drop_ul())
         FluxPoints(sed_type="flux", n_points=19)
         """
-        table_drop_ul = self.table[~self._is_ul]
+        table_drop_ul = self.table[~self.is_ul]
         return self.__class__(table_drop_ul)
 
-    def to_sed_type(self, sed_type, method='log_center', model=None, pwl_approx=False):
+    def to_sed_type(self, sed_type, method="log_center", model=None, pwl_approx=False):
         """Convert to a different SED type (return new `FluxPoints`).
 
         See: http://adsabs.harvard.edu/abs/1995NIMPA.355..541L for details
@@ -242,7 +251,7 @@ class FluxPoints(object):
         ----------
         sed_type : {'dnde'}
              SED type to convert to.
-        model : `~gammapy.spectrum.SpectralModel`
+        model : `~gammapy.spectrum.models.SpectralModel`
             Spectral model assumption.  Note that the value of the amplitude parameter
             does not matter. Still it is recommended to use something with the right
             scale and units. E.g. `amplitude = 1e-12 * u.Unit('cm-2 s-1 TeV-1')`
@@ -273,13 +282,13 @@ class FluxPoints(object):
         >>> flux_points_dnde = flux_points.to_sed_type('dnde', model=model)
         """
         # TODO: implement other directions. Refactor!
-        if sed_type != 'dnde':
+        if sed_type != "dnde":
             raise NotImplementedError
 
         if model is None:
             model = PowerLaw(
-                index=2 * u.Unit(''),
-                amplitude=1 * u.Unit('cm-2 s-1 TeV-1'),
+                index=2 * u.Unit(""),
+                amplitude=1 * u.Unit("cm-2 s-1 TeV-1"),
                 reference=1 * u.TeV,
             )
 
@@ -288,38 +297,40 @@ class FluxPoints(object):
         e_min, e_max = self.e_min, self.e_max
 
         # Compute e_ref
-        if method == 'table':
-            e_ref = input_table['e_ref'].quantity
-        elif method == 'log_center':
+        if method == "table":
+            e_ref = input_table["e_ref"].quantity
+        elif method == "log_center":
             e_ref = np.sqrt(e_min * e_max)
-        elif method == 'lafferty':
+        elif method == "lafferty":
             # set e_ref that it represents the mean dnde in the given energy bin
             e_ref = self._e_ref_lafferty(model, e_min, e_max)
         else:
-            raise ValueError('Invalid method: {}'.format(method))
+            raise ValueError("Invalid method: {}".format(method))
 
-        flux = input_table['flux'].quantity
+        flux = input_table["flux"].quantity
         dnde = self._dnde_from_flux(flux, model, e_ref, e_min, e_max, pwl_approx)
 
         # Add to result table
         table = input_table.copy()
-        table['e_ref'] = e_ref
-        table['dnde'] = dnde
+        table["e_ref"] = e_ref
+        table["dnde"] = dnde
 
-        if 'flux_err' in table.colnames:
+        if "flux_err" in table.colnames:
             # TODO: implement better error handling, e.g. MC based method
-            table['dnde_err'] = dnde * table['flux_err'].quantity / flux
+            table["dnde_err"] = dnde * table["flux_err"].quantity / flux
 
-        if 'flux_errn' in table.colnames:
-            table['dnde_errn'] = dnde * table['flux_errn'].quantity / flux
-            table['dnde_errp'] = dnde * table['flux_errp'].quantity / flux
+        if "flux_errn" in table.colnames:
+            table["dnde_errn"] = dnde * table["flux_errn"].quantity / flux
+            table["dnde_errp"] = dnde * table["flux_errp"].quantity / flux
 
-        if 'flux_ul' in table.colnames:
-            flux_ul = table['flux_ul'].quantity
-            dnde_ul = self._dnde_from_flux(flux_ul, model, e_ref, e_min, e_max, pwl_approx)
-            table['dnde_ul'] = dnde_ul
+        if "flux_ul" in table.colnames:
+            flux_ul = table["flux_ul"].quantity
+            dnde_ul = self._dnde_from_flux(
+                flux_ul, model, e_ref, e_min, e_max, pwl_approx
+            )
+            table["dnde_ul"] = dnde_ul
 
-        table.meta['SED_TYPE'] = 'dnde'
+        table.meta["SED_TYPE"] = "dnde"
         return FluxPoints(table)
 
     @staticmethod
@@ -344,8 +355,9 @@ class FluxPoints(object):
 
         if pwl_approx:
             index = model.spectral_index(e_ref)
-            flux_model = power_law_integral_flux(f=dnde_model, g=index, e=e_ref,
-                                                 e1=e_min, e2=e_max)
+            flux_model = power_law_integral_flux(
+                f=dnde_model, g=index, e=e_ref, e1=e_min, e2=e_max
+            )
         else:
             flux_model = model.integral(e_min, e_max, intervals=True)
 
@@ -357,7 +369,7 @@ class FluxPoints(object):
 
         One of: {'dnde', 'e2dnde', 'flux', 'eflux'}
         """
-        return self.table.meta['SED_TYPE']
+        return self.table.meta["SED_TYPE"]
 
     @staticmethod
     def _guess_sed_type(table):
@@ -381,20 +393,21 @@ class FluxPoints(object):
         # TODO: do we really want to error out on tables that don't have `SED_TYPE` in meta?
         # If yes, then this needs to be documented in the docstring,
         # and the workaround pointed out (to add the meta key before creating FluxPoints).
-        sed_type = table.meta['SED_TYPE']
+        sed_type = table.meta["SED_TYPE"]
         required = set(REQUIRED_COLUMNS[sed_type])
 
         if not required.issubset(table.colnames):
             missing = required.difference(table.colnames)
-            raise ValueError("Missing columns for sed type '{}':"
-                             " {}".format(sed_type, missing))
+            raise ValueError(
+                "Missing columns for sed type '{}':" " {}".format(sed_type, missing)
+            )
 
     def _get_y_energy_unit(self, y_unit):
         """Get energy part of the given y unit."""
         try:
-            return [_ for _ in y_unit.bases if _.physical_type == 'energy'][0]
+            return [_ for _ in y_unit.bases if _.physical_type == "energy"][0]
         except IndexError:
-            return u.Unit('TeV')
+            return u.Unit("TeV")
 
     def get_energy_err(self, sed_type=None):
         """Compute energy error for given sed type"""
@@ -402,8 +415,8 @@ class FluxPoints(object):
         if sed_type is None:
             sed_type = self.sed_type
         try:
-            e_min = self.table['e_min'].quantity
-            e_max = self.table['e_max'].quantity
+            e_min = self.table["e_min"].quantity
+            e_max = self.table["e_max"].quantity
             e_ref = self.e_ref
             x_err = ((e_ref - e_min), (e_max - e_ref))
         except KeyError:
@@ -416,13 +429,13 @@ class FluxPoints(object):
             sed_type = self.sed_type
         try:
             # asymmetric error
-            y_errn = self.table[sed_type + '_errn'].quantity
-            y_errp = self.table[sed_type + '_errp'].quantity
+            y_errn = self.table[sed_type + "_errn"].quantity
+            y_errp = self.table[sed_type + "_errp"].quantity
             y_err = (y_errn, y_errp)
         except KeyError:
             try:
                 # symmetric error
-                y_err = self.table[sed_type + '_err'].quantity
+                y_err = self.table[sed_type + "_err"].quantity
                 y_err = (y_err, y_err)
             except KeyError:
                 # no error at all
@@ -430,9 +443,9 @@ class FluxPoints(object):
         return y_err
 
     @property
-    def _is_ul(self):
+    def is_ul(self):
         try:
-            return self.table['is_ul'].data.astype('bool')
+            return self.table["is_ul"].data.astype("bool")
         except KeyError:
             return np.isnan(self.table[self.sed_type])
 
@@ -449,7 +462,7 @@ class FluxPoints(object):
             Reference energy.
         """
         try:
-            return self.table['e_ref'].quantity
+            return self.table["e_ref"].quantity
         except KeyError:
             return np.sqrt(self.e_min * self.e_max)
 
@@ -464,7 +477,7 @@ class FluxPoints(object):
         e_min : `~astropy.units.Quantity`
             Lower bound of energy bin.
         """
-        return self.table['e_min'].quantity
+        return self.table["e_min"].quantity
 
     @property
     def e_max(self):
@@ -477,10 +490,17 @@ class FluxPoints(object):
         e_max : `~astropy.units.Quantity`
             Upper bound of energy bin.
         """
-        return self.table['e_max'].quantity
+        return self.table["e_max"].quantity
 
-    def plot(self, ax=None, sed_type=None, energy_unit='TeV', flux_unit=None,
-             energy_power=0, **kwargs):
+    def plot(
+        self,
+        ax=None,
+        sed_type=None,
+        energy_unit="TeV",
+        flux_unit=None,
+        energy_power=0,
+        **kwargs
+    ):
         """Plot flux points.
 
         Parameters
@@ -515,7 +535,7 @@ class FluxPoints(object):
         x = self.e_ref.to(energy_unit)
 
         # get errors and ul
-        is_ul = self._is_ul
+        is_ul = self.is_ul
         x_err_all = self.get_energy_err(sed_type)
         y_err_all = self.get_flux_err(sed_type)
 
@@ -529,43 +549,53 @@ class FluxPoints(object):
         if y_err_all:
             y_errn = (y_err_all[0] * np.power(x, energy_power)).to(y_unit)
             y_errp = (y_err_all[1] * np.power(x, energy_power)).to(y_unit)
-            y_err = (y_errn[~is_ul].to(y_unit).value,
-                     y_errp[~is_ul].to(y_unit).value)
+            y_err = (y_errn[~is_ul].to(y_unit).value, y_errp[~is_ul].to(y_unit).value)
 
         if x_err_all:
             x_errn, x_errp = x_err_all
-            x_err = (x_errn[~is_ul].to(energy_unit).value,
-                     x_errp[~is_ul].to(energy_unit).value)
+            x_err = (
+                x_errn[~is_ul].to(energy_unit).value,
+                x_errp[~is_ul].to(energy_unit).value,
+            )
 
         # set flux points plotting defaults
-        kwargs.setdefault('marker', '+')
-        kwargs.setdefault('ls', 'None')
+        kwargs.setdefault("marker", "+")
+        kwargs.setdefault("ls", "None")
 
-        ebar = ax.errorbar(x[~is_ul].value, y[~is_ul].value, yerr=y_err,
-                           xerr=x_err, **kwargs)
+        ebar = ax.errorbar(
+            x[~is_ul].value, y[~is_ul].value, yerr=y_err, xerr=x_err, **kwargs
+        )
 
         if is_ul.any():
             if x_err_all:
                 x_errn, x_errp = x_err_all
-                x_err = (x_errn[is_ul].to(energy_unit).value,
-                         x_errp[is_ul].to(energy_unit).value)
+                x_err = (
+                    x_errn[is_ul].to(energy_unit).value,
+                    x_errp[is_ul].to(energy_unit).value,
+                )
 
-            y_ul = self.table[sed_type + '_ul'].quantity
+            y_ul = self.table[sed_type + "_ul"].quantity
             y_ul = (y_ul * np.power(x, energy_power)).to(y_unit)
 
             y_err = (0.5 * y_ul[is_ul].value, np.zeros_like(y_ul[is_ul].value))
 
-            kwargs.setdefault('c', ebar[0].get_color())
+            kwargs.setdefault("c", ebar[0].get_color())
 
             # pop label keyword to avoid that it appears twice in the legend
-            kwargs.pop('label', None)
-            ax.errorbar(x[is_ul].value, y_ul[is_ul].value, xerr=x_err, yerr=y_err,
-                        uplims=True, **kwargs)
+            kwargs.pop("label", None)
+            ax.errorbar(
+                x[is_ul].value,
+                y_ul[is_ul].value,
+                xerr=x_err,
+                yerr=y_err,
+                uplims=True,
+                **kwargs
+            )
 
-        ax.set_xscale('log', nonposx='clip')
-        ax.set_yscale('log', nonposy='clip')
-        ax.set_xlabel('Energy ({})'.format(energy_unit))
-        ax.set_ylabel('{} ({})'.format(self.sed_type, y_unit))
+        ax.set_xscale("log", nonposx="clip")
+        ax.set_yscale("log", nonposy="clip")
+        ax.set_xlabel("Energy ({})".format(energy_unit))
+        ax.set_ylabel("{} ({})".format(self.sed_type, y_unit))
         return ax
 
 
@@ -575,6 +605,16 @@ class FluxPointEstimator(object):
     Computes flux points for a given spectrum observation dataset
     (a 1-dim on/off observation), energy binning and spectral model.
 
+    A spectral model shape is assumed, all parameters except the norm are fixed
+    A 1D amplitude fit is done, to find the amplitude so that npred equals excess.
+
+    This is done for each group independently.
+
+    The method is used for example in this FERMI-LAT catalog paper
+    https://ui.adsabs.harvard.edu/#abs/2015ApJS..218...23A
+    or the HESS GPS paper
+    https://ui.adsabs.harvard.edu/#abs/2018A%26A...612A...1H
+
     Parameters
     ----------
     obs : `~gammapy.spectrum.SpectrumObservation` or `~gammapy.spectrum.SpectrumObservationList`
@@ -590,95 +630,65 @@ class FluxPointEstimator(object):
         self.groups = groups
         self.model = model
         self.flux_points = None
+        self._fit = None
 
     def __str__(self):
-        s = '{}:\n'.format(self.__class__.__name__)
-        s += str(self.obs) + '\n'
-        s += str(self.groups) + '\n'
-        s += str(self.model) + '\n'
+        s = "{}:\n".format(self.__class__.__name__)
+        s += str(self.obs) + "\n"
+        s += str(self.groups) + "\n"
+        s += str(self.model) + "\n"
         return s
 
+    @property
+    def fit(self):
+        """Instance of `~gammapy.spectrum.SpectrumFit`"""
+        return self._fit
+
+    @property
+    def obs(self):
+        """Observations participating in the fit"""
+        return self._obs
+
+    @obs.setter
+    def obs(self, obs):
+        if isinstance(obs, SpectrumObservation):
+            obs = SpectrumObservationList([obs])
+
+        self._obs = SpectrumObservationList(obs)
+
     def compute_points(self):
         rows = []
         for group in self.groups:
-            if group.bin_type != 'normal':
-                log.debug('Skipping energy group:\n{}'.format(group))
+            if group.bin_type != "normal":
+                log.debug("Skipping energy group:\n{}".format(group))
                 continue
 
             row = self.compute_flux_point(group)
             rows.append(row)
 
-        meta = OrderedDict([
-            ('method', 'TODO'),
-            ('SED_TYPE', 'dnde'),
-        ])
+        meta = OrderedDict([("method", "TODO"), ("SED_TYPE", "dnde")])
         table = table_from_row_data(rows=rows, meta=meta)
         self.flux_points = FluxPoints(table)
 
     def compute_flux_point(self, energy_group):
-        log.debug('Computing flux point for energy group:\n{}'.format(energy_group))
+        log.debug("Computing flux point for energy group:\n{}".format(energy_group))
 
-        model = self.compute_approx_model(
-            global_model=self.model,
-            energy_group=energy_group,
-        )
+        model = self._get_spectral_model(self.model)
 
         # Put at log center of the bin
         energy_ref = np.sqrt(energy_group.energy_min * energy_group.energy_max)
 
         return self.fit_point(
-            model=model,
-            energy_group=energy_group,
-            energy_ref=energy_ref,
+            model=model, energy_group=energy_group, energy_ref=energy_ref
         )
 
     @staticmethod
-    def compute_approx_model(global_model, energy_group):
-        """
-        Compute approximate model, to be used in the energy bin.
-        TODO: At the moment just the global model with fixed parameters is
-        returned
-        """
-        # binning = EnergyBounds(binning)
-        # low_bins = binning.lower_bounds
-        # high_bins = binning.upper_bounds
-        #
-        # from sherpa.models import PowLaw1D
-        #
-        # if isinstance(model, models.PowerLaw):
-        #     temp = model.to_sherpa()
-        #     temp.gamma.freeze()
-        #     sherpa_models = [temp] * binning.nbins
-        # else:
-        #     sherpa_models = [None] * binning.nbins
-        #
-        # for low, high, sherpa_model in zip(low_bins, high_bins, sherpa_models):
-        #     log.info('Computing flux points in bin [{}, {}]'.format(low, high))
-        #
-        #     # Make PowerLaw approximation for higher order models
-        #     if sherpa_model is None:
-        #         flux_low = model(low)
-        #         flux_high = model(high)
-        #         index = powerlaw.power_law_g_from_points(e1=low, e2=high,
-        #                                                  f1=flux_low,
-        #                                                  f2=flux_high)
-        #
-        #         log.debug('Approximated power law index: {}'.format(index))
-        #         sherpa_model = PowLaw1D('powlaw1d.default')
-        #         sherpa_model.gamma = index
-        #         sherpa_model.gamma.freeze()
-        #         sherpa_model.ref = model.parameters.reference.to('keV')
-        #         sherpa_model.ampl = 1e-20
-        # return PowerLaw(
-        #    index=u.Quantity(2, ''),
-        #    amplitude=u.Quantity(1, 'm-2 s-1 TeV-1'),
-        #    reference=u.Quantity(1, 'TeV'),
-        # )
-        approx_model = global_model.copy()
-        for par in approx_model.parameters.parameters:
-            if par.name != 'amplitude':
+    def _get_spectral_model(global_model):
+        model = global_model.copy()
+        for par in model.parameters.parameters:
+            if par.name != "amplitude":
                 par.frozen = True
-        return approx_model
+        return model
 
     def compute_flux_point_ul(self, fit, stat_best_fit, delta_ts=4, negative=False):
         """
@@ -700,7 +710,7 @@ class FluxPointEstimator(object):
 
         Examples
         --------
-        To compute ~95% confidence upper limits (or 2 sigma) you can use:
+        To compute ~95% confidence upper limits (or 2 sigma) you can use::
 
             from scipy.stats import chi2, norm
 
@@ -708,20 +718,18 @@ class FluxPointEstimator(object):
             cl = 1 - 2 * norm.sf(sigma) # using two sided p-value
             delta_ts = chi2.isf(1 - cl, df=1)
 
-
         Returns
         -------
         dnde_ul : `~astropy.units.Quantity`
             Flux point upper limit.
-
         """
-
         from scipy.optimize import brentq
 
+        model = fit.result[0].model.copy()
         # this is a prototype for fast flux point upper limit
         # calculation using brentq
-        amplitude = fit.result[0].model.parameters['amplitude'].value / 1E-12
-        amplitude_err = fit.result[0].model.parameters.error('amplitude') / 1E-12
+        amplitude = model.parameters["amplitude"].value / 1E-12
+        amplitude_err = model.parameters.error("amplitude") / 1E-12
 
         if negative:
             amplitude_max = amplitude
@@ -731,20 +739,20 @@ class FluxPointEstimator(object):
             amplitude_min = amplitude
 
         def ts_diff(x):
-            fit.model.parameters['amplitude'].value = x * 1E-12
-            fit.predict_counts()
-            fit.calc_statval()
-            return (stat_best_fit + delta_ts) - fit.total_stat
+            model.parameters["amplitude"].value = x * 1E-12
+            stat = fit.total_stat(model.parameters)
+            return (stat_best_fit + delta_ts) - stat
 
         try:
-            result = brentq(ts_diff, amplitude_min, amplitude_max,
-                            maxiter=100, rtol=1e-2)
-            fit.model.parameters['amplitude'].value = result * 1E-12
-            return fit.model(fit.model.parameters['reference'].quantity)
+            result = brentq(
+                ts_diff, amplitude_min, amplitude_max, maxiter=100, rtol=1e-2
+            )
+            model.parameters["amplitude"].value = result * 1E-12
+            return model(model.parameters["reference"].quantity)
         except (RuntimeError, ValueError):
             # Where the root finding fails NaN is set as amplitude
-            log.debug('Flux point upper limit computation failed.')
-            return np.nan * u.Unit(fit.model.parameters['amplitude'].unit)
+            log.debug("Flux point upper limit computation failed.")
+            return np.nan * u.Unit(model.parameters["amplitude"].unit)
 
     def compute_flux_point_sqrt_ts(self, fit, stat_best_fit):
         """
@@ -765,17 +773,16 @@ class FluxPointEstimator(object):
             Sqrt(TS) for flux point.
 
         """
+        model = fit.result[0].model.copy()
         # store best fit amplitude, set amplitude of fit model to zero
-        amplitude = fit.result[0].model.parameters['amplitude'].value
-        fit.model.parameters['amplitude'].value = 0
+        amplitude = model.parameters["amplitude"].value
 
         # determine TS value for amplitude zero
-        fit.predict_counts()
-        fit.calc_statval()
-        stat_null = fit.total_stat
+        model.parameters["amplitude"].value = 0
+        stat_null = fit.total_stat(model.parameters)
 
         # set amplitude of fit model to best fit amplitude
-        fit.model.parameters['amplitude'].value = amplitude
+        model.parameters["amplitude"].value = amplitude
 
         # compute sqrt TS
         ts = np.abs(stat_null - stat_best_fit)
@@ -787,44 +794,69 @@ class FluxPointEstimator(object):
         energy_min = energy_group.energy_min
         energy_max = energy_group.energy_max
 
+        quality_orig = []
+
+        for index in range(len(self.obs)):
+            # Set quality bins to only bins in energy_group
+            quality_orig_unit = self.obs[index].on_vector.quality
+            quality_len = len(quality_orig_unit)
+            quality_orig.append(quality_orig_unit)
+            quality = np.zeros(quality_len, dtype=int)
+            for bin in range(quality_len):
+                if (
+                    (bin < energy_group.bin_idx_min)
+                    or (bin > energy_group.bin_idx_max)
+                    or (energy_group.bin_type != "normal")
+                ):
+                    quality[bin] = 1
+            self.obs[index].on_vector.quality = quality
+
         # Set reference and remove min amplitude
-        model.parameters['reference'].value = energy_ref.to('TeV').value
+        model.parameters["reference"].value = energy_ref.to("TeV").value
 
-        fit = SpectrumFit(self.obs, model)
+        self._fit = SpectrumFit(self.obs, model)
 
-        # TODO: Notice channels contained in energy_group
-        fit.fit_range = energy_min, energy_max
+        for index in range(len(quality_orig)):
+            self.obs[index].on_vector.quality = quality_orig[index]
 
         log.debug(
-            'Calling Sherpa fit for flux point '
-            ' in energy range:\n{}'.format(fit)
+            "Calling Sherpa fit for flux point "
+            " in energy range:\n{}".format(self.fit)
         )
 
-        fit.fit()
-        fit.est_errors()
+        result = self.fit.run()
 
         # compute TS value for all observations
-        stat_best_fit = np.sum([res.statval for res in fit.result])
+        stat_best_fit = result.total_stat
 
-        dnde, dnde_err = fit.result[0].model.evaluate_error(energy_ref)
-        sqrt_ts = self.compute_flux_point_sqrt_ts(fit, stat_best_fit=stat_best_fit)
+        dnde, dnde_err = self.fit.result[0].model.evaluate_error(energy_ref)
+        sqrt_ts = self.compute_flux_point_sqrt_ts(self.fit, stat_best_fit=stat_best_fit)
 
-        dnde_ul = self.compute_flux_point_ul(fit, stat_best_fit=stat_best_fit)
-        dnde_errp = self.compute_flux_point_ul(fit, stat_best_fit=stat_best_fit, delta_ts=1.) - dnde
-        dnde_errn = dnde - self.compute_flux_point_ul(fit, stat_best_fit=stat_best_fit, delta_ts=1., negative=True)
+        dnde_ul = self.compute_flux_point_ul(self.fit, stat_best_fit=stat_best_fit)
+        dnde_errp = (
+            self.compute_flux_point_ul(
+                self.fit, stat_best_fit=stat_best_fit, delta_ts=1.
+            )
+            - dnde
+        )
+        dnde_errn = dnde - self.compute_flux_point_ul(
+            self.fit, stat_best_fit=stat_best_fit, delta_ts=1., negative=True
+        )
 
-        return OrderedDict([
-            ('e_ref', energy_ref),
-            ('e_min', energy_min),
-            ('e_max', energy_max),
-            ('dnde', dnde.to(DEFAULT_UNIT['dnde'])),
-            ('dnde_err', dnde_err.to(DEFAULT_UNIT['dnde'])),
-            ('dnde_ul', dnde_ul.to(DEFAULT_UNIT['dnde'])),
-            ('is_ul', sqrt_ts < sqrt_ts_threshold),
-            ('sqrt_ts', sqrt_ts),
-            ('dnde_errp', dnde_errp),
-            ('dnde_errn', dnde_errn)
-        ])
+        return OrderedDict(
+            [
+                ("e_ref", energy_ref),
+                ("e_min", energy_min),
+                ("e_max", energy_max),
+                ("dnde", dnde.to(DEFAULT_UNIT["dnde"])),
+                ("dnde_err", dnde_err.to(DEFAULT_UNIT["dnde"])),
+                ("dnde_ul", dnde_ul.to(DEFAULT_UNIT["dnde"])),
+                ("is_ul", sqrt_ts < sqrt_ts_threshold),
+                ("sqrt_ts", sqrt_ts),
+                ("dnde_errp", dnde_errp),
+                ("dnde_errn", dnde_errn),
+            ]
+        )
 
 
 class FluxPointProfiles(object):
@@ -852,49 +884,17 @@ class FluxPointProfiles(object):
         table = Table.read(str(filename), **kwargs)
         return cls(table=table)
 
-    def plot(self, ax):
-        # TODO: copy code from fermipy, don't start from scratch!
-        raise NotImplementedError
 
-
-def chi2_flux_points(flux_points, gp_model):
-    """
-    Chi2 statistics for a list of flux points and model.
-    """
-    model = gp_model(flux_points.e_ref)
-    data = flux_points.table['dnde'].quantity
-    data_err = flux_points.table['dnde_err'].quantity
-    stat_per_bin = ((data - model) / data_err).value ** 2
-    return np.nansum(stat_per_bin), stat_per_bin
-
-
-def chi2_flux_points_assym(flux_points, gp_model):
-    """
-    Assymetric chi2 statistics for a list of flux points and model.
-    """
-    model = gp_model(flux_points.e_ref)
-    data = flux_points.table['dnde'].quantity
-
-    data_errp = flux_points.table['dnde_errp'].quantity
-    data_errn = flux_points.table['dnde_errn'].quantity
-
-    data_err = np.where(model > data, data_errp, data_errn)
-    stat_per_bin = ((data - model) / data_err).value ** 2
-    return np.nansum(stat_per_bin), stat_per_bin
-
-
-class FluxPointFitter(object):
+class FluxPointFit(Fit):
     """
     Fit a set of flux points with a parametric model.
 
     Parameters
     ----------
-    optimizer : {'simplex', 'moncar', 'gridsearch'}
-        Select optimizer
-    error_estimator : {'covar'}
-        Select error estimator
-    ul_handling : {'ignore'}
-        How to handle flux point upper limits in the fit
+    model : `~gammapy.spectrum.models.SpectralModel`
+        Spectral model (with fit start parameters)
+    data : `~gammapy.spectrum.FluxPoints`
+        Flux points.
 
     Examples
     --------
@@ -902,7 +902,7 @@ class FluxPointFitter(object):
     Load flux points from file and fit with a power-law model::
 
         from astropy import units as u
-        from gammapy.spectrum import FluxPoints, FluxPointFitter
+        from gammapy.spectrum import FluxPoints, FluxPointFit
         from gammapy.spectrum.models import PowerLaw
 
         filename = '$GAMMAPY_EXTRA/test_datasets/spectrum/flux_points/diff_flux_points.fits'
@@ -914,136 +914,53 @@ class FluxPointFitter(object):
             reference=1. * u.TeV,
         )
 
-        fitter = FluxPointFitter()
-        result = fitter.run(flux_points, model)
+        fitter = FluxPointFit(model, flux_points)
+        result = fitter.run()
         print(result['best_fit_model'])
     """
 
-    def __init__(self, stat='chi2', optimizer='simplex', error_estimator='covar',
-                 ul_handling='ignore'):
-        if stat == 'chi2':
-            self.stat = chi2_flux_points
-        elif stat == 'chi2assym':
-            self.stat = chi2_flux_points_assym
+    def __init__(self, model, data, stat="chi2"):
+        self._model = model.copy()
+        self.data = data
+
+        if stat in ["chi2", "chi2assym"]:
+            self._stat = stat
         else:
-            raise ValueError("'{stat}' is not a valid fit statistic, please choose"
-                             " either 'chi2' or 'chi2assym'")
+            raise ValueError(
+                "'{stat}' is not a valid fit statistic, please choose"
+                " either 'chi2' or 'chi2assym'"
+            )
 
-        if ul_handling != 'ignore':
-            raise NotImplementedError('No handling of upper limits implemented.')
+    @property
+    def _stat_chi2(self):
+        """Likelihood per bin given the current model parameters"""
+        model = self._model(self.data.e_ref)
+        data = self.data.table["dnde"].quantity
+        sigma = self.data.table["dnde_err"].quantity
+        return ((data - model) / sigma).to("").value ** 2
 
-        self.parameters = OrderedDict([
-            ('optimizer', optimizer),
-            ('error_estimator', error_estimator),
-            ('ul_handling', ul_handling),
-        ])
+    @property
+    def _stat_chi2_assym(self):
+        """
+        Assymetric chi2 statistics for a list of flux points and model.
+        """
+        model = self._model(self.data.e_ref)
+        data = self.data.table["dnde"].quantity.to(model.unit)
+        data_errp = self.data.table["dnde_errp"].quantity
+        data_errn = self.data.table["dnde_errn"].quantity
 
-    def _setup_sherpa_fit(self, data, model):
-        """Fit flux point using sherpa"""
-        from sherpa.fit import Fit
-        from sherpa.data import DataSimulFit
-        from ..utils.sherpa import (SherpaDataWrapper, SherpaStatWrapper,
-                                    SherpaModelWrapper, SHERPA_OPTMETHODS)
+        val_n = ((data - model) / data_errn).to("").value ** 2
+        val_p = ((data - model) / data_errp).to("").value ** 2
+        return np.where(model > data, val_p, val_n)
 
-        optimizer = self.parameters['optimizer']
-
-        if data.sed_type == 'dnde':
-            data = SherpaDataWrapper(data)
+    @property
+    def stat(self):
+        if self._stat == "chi2":
+            return self._stat_chi2
         else:
-            raise NotImplementedError('Only fitting of differential flux points data '
-                                      'is supported.')
+            return self._stat_chi2_assym
 
-        stat = SherpaStatWrapper(self.stat)
-        data = DataSimulFit(name='GPFluxPoints', datasets=[data])
-        method = SHERPA_OPTMETHODS[optimizer]
-        models = SherpaModelWrapper(model)
-        return Fit(data=data, model=models, stat=stat, method=method)
-
-    def fit(self, data, model):
-        """
-        Fit given model to data.
-
-        Parameters
-        ----------
-        model : `~gammapy.spectrum.models.SpectralModel`
-            Spectral model (with fit start parameters)
-
-        Returns
-        -------
-        best_fit_model : `~gammapy.spectrum.models.SpectralModel`
-            Best fit model
-        """
-        p = self.parameters
-        model = model.copy()
-
-        if p['optimizer'] in ['simplex', 'moncar', 'gridsearch', 'levmar']:
-            sherpa_fitter = self._setup_sherpa_fit(data, model)
-            sherpa_fitter.fit()
-        else:
-            raise ValueError('Not a valid optimizer')
-
-        return model
-
-    def statval(self, data, model):
-        """
-        Compute statval for given model and data.
-
-        Parameters
-        ----------
-        model : `~gammapy.spectrum.models.SpectralModel`
-            Spectral model
-        """
-        return self.stat(data, model)
-
-    def dof(self, data, model):
-        """
-        Degrees of freedom.
-
-        Parameters
-        ----------
-        model : `~gammapy.spectrum.models.SpectralModel`
-            Spectral model
-        """
-        m = len(model.parameters.free)
-        n = len(data.table)
-        return n - m
-
-    def estimate_errors(self, data, model):
-        """
-        Estimate errors on best fit parameters.
-        """
-        model = model.copy()
-        sherpa_fitter = self._setup_sherpa_fit(data, model)
-        result = sherpa_fitter.est_errors()
-        covariance = result.extra_output
-        covar_axis = model.parameters.free
-        model.parameters.set_parameter_covariance(covariance, covar_axis)
-        return model
-
-    def run(self, data, model):
-        """
-        Run all fitting adn extra information steps.
-
-        Parameters
-        ----------
-        data : list of `~gammapy.spectrum.FluxPoints`
-            Flux points.
-        model : `~gammapy.spectrum.models.SpectralModel`
-            Spectral model
-
-        Returns
-        -------
-        result : `~collections.OrderedDict`
-            Dictionary with fit results and debug output.
-        """
-        best_fit_model = self.fit(data, model)
-        best_fit_model = self.estimate_errors(data, best_fit_model)
-        dof = self.dof(data, best_fit_model)
-        statval = self.statval(data, best_fit_model)[0]
-
-        return OrderedDict([
-            ('best-fit-model', best_fit_model),
-            ('dof', int(dof)),
-            ('statval', float(statval)),
-            ('statval/dof', float(statval / dof)),
-        ])
+    def total_stat(self, parameters):
+        """Total likelihood given the current model parameters"""
+        self._model.parameters = parameters
+        return np.nansum(self.stat, dtype=np.float64)
