@@ -5,6 +5,6 @@ from ..units import standardise_unit
 
 
 def test_standardise_unit():
-    assert standardise_unit('ph cm-2 s-1') == u.Unit('cm-2 s-1')
-    assert standardise_unit('ct cm-2 s-1') == u.Unit('cm-2 s-1')
-    assert standardise_unit('cm-2 s-1') == u.Unit('cm-2 s-1')
+    assert standardise_unit("ph cm-2 s-1") == u.Unit("cm-2 s-1")
+    assert standardise_unit("ct cm-2 s-1") == u.Unit("cm-2 s-1")
+    assert standardise_unit("cm-2 s-1") == u.Unit("cm-2 s-1")
